#define F_CPU 16000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#define ADC_AREF_VOLTAGE 3.3
#define BLUE 0
#define CYAN 1
#define GREEN 2
#define YELLOW 3
#define ORANGE 4
#define MAGENTA 5
#define RED 6
#define ADC_LIMIT 0.5

int colors[6] = {YELLOW, ORANGE, MAGENTA, RED, CYAN, GREEN};

void InitADC()
{
	ADMUX = 0;
	ADMUX |= (1 << MUX0);          // ADC1 - channel 1
	ADMUX |= (1 << REFS0);         // AVCC with external capacitor at AREF pin

	ADCSRA = 0;
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); //set clock prescaler at 128
	ADCSRA |= (1 << ADEN);         // enable ADC
}

uint16_t ReadADC()
{
	ADCSRA |= (1 << ADSC);         //start conversion
	while(ADCSRA & (1<<ADSC));
    	return (ADC);
}

double ADC_voltage(int raw)
{
    (void)raw;
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}

void TIMER1_init()
{
    TCCR1B = (5 << CS10) | (1 << WGM12);
    TIMSK1 |= (1 << OCIE1A);
    OCR1A = 1561;
}

void SetColor(int color)
{
	switch(color) {
		case MAGENTA:
			PORTD &= ~(1 << PD4);
			PORTD |= (1 << PD5);
			PORTD &= ~(1 << PD7);
			break;
		case BLUE:
			PORTD &= ~(1 << PD4);
			PORTD |= (1 << PD5);
			PORTD |= (1 << PD7);
			break;
		case CYAN:
			PORTD &= ~(1 << PD4);
			PORTD &= ~(1 << PD5);
			PORTD |= (1 << PD7);
			break;
		case GREEN:
			PORTD |= (1 << PD4);
			PORTD &= ~(1 << PD5);
			PORTD |= (1 << PD7);
			break;
		case YELLOW:
			PORTD |= (1 << PD4);
			PORTD &= ~(1 << PD5);
		 	PORTD &= ~(1 << PD7);
			break;
		case ORANGE:
			PORTD |= (1 << PD4);
			PORTD |= (1 << PD5);
			PORTD |= (1 << PD7);
			break;
		case RED:
			PORTD |= (1 << PD4);
			PORTD |= (1 << PD5);
			PORTD &= ~(1 << PD7);
			break;
	}
}

int returnColor() {
	int r = rand() % 6;
	return colors[r];
}

void CheckADC(double value)
{
	if (value < ADC_LIMIT)
		SetColor(returnColor());
	else
		SetColor(BLUE);
}

int main()
{
	
	double adc_result;

	//Set DDRD as output	
	DDRD |= (1 << PD4);
	DDRD |= (1 << PD5);
	DDRD |= (1 << PD7);	

	//Initialize ADC
   	InitADC();

	// Init Timer1
    	TIMER1_init();

	_delay_ms(50);

	while(1) {
		adc_result = ADC_voltage(ReadADC());
		CheckADC(adc_result);

		_delay_ms(50);
	} 	
	return 0;
}

